Widget.onPropertyChange = function (propName, newValue, oldValue) {
    var _a = Widget.props, value = _a.value, step_value = _a.step_value, required = _a.required, min_value = _a.min_value, max_value = _a.max_value, show_labels = _a.show_labels, show_icons = _a.show_icons, icon_actions = _a.icon_actions, start_icon = _a.start_icon, end_icon = _a.end_icon;
    var widgetMap = Widget.Widgets;
    var showIconsBool = convertToBoolean(show_icons);
    var showLabelsBool = convertToBoolean(show_labels);
    var iconActionsBool = convertToBoolean(icon_actions);
    switch (propName) {
        case 'prop-value':
        case 'prop-min_value':
        case 'prop-max_value':
            updateSlider(widgetMap, value, min_value, max_value, step_value);
            break;
        case 'prop-step_value':
            updateSliderStep(widgetMap, value, min_value, max_value, step_value);
            break;
        case 'prop-required':
            setRequired(widgetMap, required);
            break;
        case "prop-show_labels":
            toggleLabelVisibilty(showLabelsBool, widgetMap);
            break;
        case "prop-show_icons":
            toggleIconsVisibility(showIconsBool, iconActionsBool, widgetMap);
            break;
        case "prop-start_icon":
            updateStartIconVisibility(iconActionsBool, widgetMap);
            break;
        case "prop-end_icon":
            updateEndIconVisibility(iconActionsBool, widgetMap);
            break;
    }
};
Widget.onReady = function () {
    var _a = Widget.props, value = _a.value, step_value = _a.step_value, required = _a.required, min_value = _a.min_value, max_value = _a.max_value, show_labels = _a.show_labels, show_icons = _a.show_icons, icon_actions = _a.icon_actions, start_icon = _a.start_icon, end_icon = _a.end_icon;
    var widgetMap = Widget.Widgets;
    var showIconsBool = convertToBoolean(show_icons);
    var showLabelsBool = convertToBoolean(show_labels);
    var iconActionsBool = convertToBoolean(icon_actions);
    widgetMap.sliderContainer.nativeElement.classList.add('filled-slider', 'discrete-slider');
    initializeSlider(widgetMap, value, min_value, max_value, step_value, required);
    toggleLabelVisibilty(showLabelsBool, widgetMap);
    toggleIconsVisibility(showIconsBool, iconActionsBool, widgetMap);
};
function initializeSlider(widgetMap, value, minValue, maxValue, stepValue, required) {
    updateSlider(widgetMap, value, minValue, maxValue, stepValue);
    setRequired(widgetMap, required);
    appendSliderIndicator(widgetMap.slider);
}
function updateSlider(widgetMap, value, minValue, maxValue, stepValue) {
    if (value !== undefined) {
        widgetMap.slider.datavalue = Number(value);
    }
    configureSliderAndSetStyles(widgetMap, value, minValue, maxValue, stepValue);
    setupStepsAndLabels(widgetMap);
}
function updateSliderStep(widgetMap, value, minValue, maxValue, stepValue) {
    updateSlider(widgetMap, value, minValue, maxValue, stepValue);
}
function setRequired(widgetMap, required) {
    widgetMap.composite.required = required;
}
function appendSliderIndicator(slider) {
    var sliderElement = slider.nativeElement;
    if (sliderElement) {
        var indicator = document.createElement('p');
        var labelContainer = document.createElement('div');
        labelContainer.className = 'stop-indicator';
        indicator.className = 'active-slider';
        sliderElement.appendChild(indicator);
        sliderElement.appendChild(labelContainer);
    }
}
function configureSliderAndSetStyles(widgetMap, value, minValue, maxValue, stepValue) {
    var slider = widgetMap.slider;
    minValue = parseInt(minValue, 10);
    maxValue = parseInt(maxValue, 10);
    stepValue = parseInt(stepValue, 10);
    if (isNaN(minValue) || isNaN(maxValue) || isNaN(stepValue) || stepValue <= 0) {
        console.error("Invalid min, max, or step values");
        return;
    }
    slider.min = minValue;
    slider.max = maxValue;
    slider.step = stepValue;
    slider.datavalue = clamp(slider.datavalue, minValue, maxValue);
    updateSliderStyles(slider, slider.datavalue, minValue, maxValue);
}
function updateSliderStyles(slider, value, minValue, maxValue) {
    var percentage = getSliderPercentage(value, minValue, maxValue);
    slider.nativeElement.style.setProperty("--end", "".concat(100 - percentage, "%"));
}
function setupStepsAndLabels(widgetMap) {
    var slider = widgetMap.slider;
    var labelContainer = widgetMap.labelsContainer;
    var minValue = parseInt(slider.min, 10);
    var maxValue = parseInt(slider.max, 10);
    var stepValue = parseInt(slider.step, 10);
    if (!labelContainer)
        return;
    var numSteps = Math.ceil((maxValue - minValue) / stepValue);
    labelContainer.$element[0].innerHTML = ''; // Clear existing labels
    for (var i = 0; i <= numSteps; i++) {
        var labelValue = minValue + i * stepValue;
        if (labelValue > maxValue)
            break;
        var stepLabel = createStepLabel(labelValue, minValue, maxValue);
        labelContainer.$element[0].appendChild(stepLabel);
    }
    slider.$element[0].addEventListener('input', function () { return updateLabelVisibility(widgetMap); });
    updateLabelVisibility(widgetMap);
}
function createStepLabel(labelValue, minValue, maxValue) {
    var stepLabel = document.createElement('p');
    stepLabel.className = 'step-label';
    stepLabel.textContent = labelValue;
    var position = ((labelValue - minValue) / (maxValue - minValue)) * 100;
    stepLabel.style.setProperty('--position', "".concat(position - 1, "%"));
    stepLabel.setAttribute('data-position', position);
    return stepLabel;
}
function updateLabelVisibility(widgetMap) {
    var slider = widgetMap.slider;
    var labels = widgetMap.labelsContainer.nativeElement.getElementsByClassName('step-label');
    var sliderValue = slider.datavalue;
    var minValue = slider.min;
    var maxValue = slider.max;
    var stepValue = slider.step;
    var validValues = Array.from({
        length: Math.ceil((maxValue - minValue) / stepValue) + 1
    }, function (_, i) { return minValue + i * stepValue; })
        .filter(function (value) { return value <= maxValue; });
    for (var _i = 0, labels_1 = labels; _i < labels_1.length; _i++) {
        var label = labels_1[_i];
        var labelValue = parseInt(label.textContent, 10);
        label.classList.remove('hidden', 'inactive', 'active');
        if (validValues.includes(labelValue)) {
            if (labelValue === sliderValue) {
                label.classList.add('hidden');
            }
            else if (labelValue < sliderValue) {
                label.classList.add('inactive');
            }
            else {
                label.classList.add('active');
            }
        }
        label.style.display = 'block';
    }
}
function getSliderPercentage(value, minValue, maxValue) {
    return ((value - minValue) / (maxValue - minValue)) * 100;
}
function clamp(value, minValue, maxValue) {
    return Math.min(Math.max(value, minValue), maxValue);
}
function toggleLabelVisibilty(showLabelsBool, widgetMap) {
    if (showLabelsBool) {
        widgetMap.slider.nativeElement.classList.remove("hide-labels");
    }
    else {
        widgetMap.slider.nativeElement.classList.add("hide-labels");
    }
}
function toggleIconsVisibility(showIconsBool, iconActionsBool, widgetMap) {
    var decrementButton = widgetMap.decrementButton, incrementButton = widgetMap.incrementButton;
    if (showIconsBool) {
        updateStartIconVisibility(iconActionsBool, widgetMap);
        updateEndIconVisibility(iconActionsBool, widgetMap);
    }
    else {
        decrementButton.show = false;
        incrementButton.show = false;
    }
}
function updateStartIconVisibility(iconActionsBool, widgetMap) {
    var decrementButton = widgetMap.decrementButton;
    decrementButton.show = true;
    decrementButton.nativeElement.style.pointerEvents = iconActionsBool ?
        "auto" :
        "none";
}
function updateEndIconVisibility(iconActionsBool, widgetMap) {
    var incrementButton = widgetMap.incrementButton;
    incrementButton.show = true;
    incrementButton.nativeElement.style.pointerEvents = iconActionsBool ?
        "auto" :
        "none";
}
function moveSlider(direction, widgetMap) {
    var slider = widgetMap.slider;
    var newValue = Number(slider.datavalue) + (direction * slider.step);
    newValue = Math.max(Math.min(newValue, slider.maxvalue), slider.minvalue);
    slider.datavalue = newValue;
    Widget.sliderChange(null, slider, newValue);
}
function convertToBoolean(value) {
    return value === true || value === "true";
}
Widget.sliderChange = function ($event, widget, newVal) {
    updateSliderStyles(widget, newVal, widget.min, widget.max);
    updateLabelVisibility(Widget.Widgets);
};
Widget.decrementButtonClick = function ($event, widget) {
    moveSlider(-1, Widget.Widgets);
};
Widget.incrementButtonClick = function ($event, widget) {
    moveSlider(1, Widget.Widgets);
};
