/*
 * Use App.getDependency for Dependency Injection
 * Example:
 * var DialogService = App.getDependency('DialogService');
 */
Widget.onPropertyChange = function (propname, newvalue, oldvalue) {
    var widgetMap = Widget.Widgets;
    var _a = Widget.props, switch_type = _a.switch_type, icon = _a.icon, unselected_icon = _a.unselected_icon, required = _a.required, unchecked_value = _a.unchecked_value, checked_value = _a.checked_value, selected = _a.selected;
    switch (propname) {
        case 'prop-switch_type':
            handleTypeChange(switch_type, icon, unselected_icon, widgetMap);
            break;
        case 'prop-required':
            handleRequiredChange(required, widgetMap);
            break;
        case 'prop-selected':
            handleSelectedValueChange(selected, widgetMap);
            break;
        case 'prop-checked_value':
            handleCheckedValueChange(checked_value, selected, widgetMap);
            break;
        case 'prop-unchecked_value':
            handleUnCheckedValueChange(unchecked_value, selected, widgetMap);
            break;
        case 'prop-icon':
            handleIconVisibiltyChange(icon, widgetMap);
            break;
    }
};
/* Perform any action on widgets/variables within this block */
Widget.onReady = function () {
    var widgetMap = Widget.Widgets;
    var _a = Widget.props, switch_type = _a.switch_type, icon = _a.icon, unselected_icon = _a.unselected_icon, required = _a.required, unchecked_value = _a.unchecked_value, checked_value = _a.checked_value, selected = _a.selected;
    handleTypeChange(switch_type, icon, unselected_icon, widgetMap);
    handleRequiredChange(required, widgetMap);
    handleCheckedValueChange(checked_value, selected, widgetMap);
    handleUnCheckedValueChange(unchecked_value, selected, widgetMap);
    handleSelectedValueChange(selected, widgetMap);
};
function handleTypeChange(switch_type, icon, unselected_icon, widgetMap) {
    widgetMap.toggle.nativeElement.classList.remove('filled-switch', 'icon', 'unselected-icon', 'condensed-switch', 'ios-switch');
    // Set CSS classes for the toggle widget based on the switch type
    if (switch_type === "Filled") {
        if (icon.toString().toLowerCase() === 'true') {
            widgetMap.toggle.nativeElement.classList.add('filled-switch', 'icon');
        }
        else {
            widgetMap.toggle.nativeElement.classList.add('filled-switch');
        }
        // Add 'unselected-icon' class if no unselected_icon is specified
        if (unselected_icon == null || unselected_icon === "" || unselected_icon === " ") {
            widgetMap.toggle.nativeElement.classList.add('filled-switch', 'icon', 'unselected-icon');
        }
    }
    else if (switch_type === "Condensed") {
        // Add 'condensed-switch' class
        widgetMap.toggle.nativeElement.classList.add('condensed-switch');
    }
    else if (switch_type === "IOS") {
        // Add 'iOS-switch' class
        widgetMap.toggle.nativeElement.classList.add('ios-switch');
    }
}
function handleRequiredChange(required, widgetMap) {
    // Set the 'required' property for the composite widget
    widgetMap.composite.required = required;
}
function handleCheckedValueChange(checked_value, selected, widgetMap) {
    // Set the 'checkedvalue' and 'uncheckedvalue' properties for the toggle widget
    widgetMap.toggle.checkedvalue = checked_value;
    handleSelectedValueChange(selected, widgetMap);
}
function handleIconVisibiltyChange(icon, widgetMap) {
    if (icon.toString().toLowerCase() === 'true') {
        widgetMap.toggle.nativeElement.classList.add('icon');
    }
    else {
        widgetMap.toggle.nativeElement.classList.remove('icon');
    }
}
function handleUnCheckedValueChange(unchecked_value, selected, widgetMap) {
    widgetMap.toggle.uncheckedvalue = unchecked_value;
    handleSelectedValueChange(selected, widgetMap);
}
function handleSelectedValueChange(selected, widgetMap) {
    // Retrieve the selected value from page parameters
    var selectedValue = selected;
    // Determine the correct 'datavalue' based on the selected value
    if (selectedValue === widgetMap.toggle.checkedvalue) {
        // If selected value matches the checked value, set datavalue to checked value
        widgetMap.toggle.datavalue = widgetMap.toggle.checkedvalue;
    }
    else if (selectedValue === widgetMap.toggle.uncheckedvalue) {
        // If selected value matches the unchecked value, set datavalue to unchecked value
        widgetMap.toggle.datavalue = widgetMap.toggle.uncheckedvalue;
    }
    else {
        // Log a warning if the selected value does not match known values
        console.warn('Selected value does not match any known value');
        // Optionally set to a default value or handle the error
        widgetMap.toggle.datavalue = widgetMap.toggle.uncheckedvalue; // Default to unchecked
    }
}
